@@ -6,13 +6,20 @@ import json
 import os
 import ssl
 import time
+import threading
+import queue
+from playsound import playsound
+from time import sleep
+import os
+if os.name == 'nt':
+    import winsound
 
 ssl._create_default_https_context = ssl._create_unverified_context
 import pandas as pd
 from termcolor import cprint
 from inspect import currentframe
 
-__version__ = '0.1.22'
+__version__ = '0.1.25'
 
 __info__ = {
     '2022/01/04: 0.1.18 加入 __info__。',
@@ -21,9 +28,15 @@ __info__ = {
     '2022/01/06: 0.1.20 add get_stock_future_data(). 用來抓取每天股票期貨資料',
     '2022/01/06: 0.1.20 add get_stock_future_snapshot(). 用來抓每天股票、股票期貨漲停、跌停價格',
     '2022/01/11: 0.1.21 Add virtual function, check_place_cover() in ti class',
-    '2022/02/10: 0.1.22 Add build_dealer_downloader()'
+    '2022/02/10: 0.1.22 Add build_dealer_downloader()',
+    '2022/02/14: 0.1.23 把 libexcel.py 搬到這邊來，以後可以全部沿用。',
+    '2022/06/13: 0.1.24 加入 tplaysound '
 }
 
+_MYPYLIB_ROOT = os.path.abspath(os.path.dirname(__file__))
+
+
+
 request_headers = {
     'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'
 }
@@ -487,7 +500,6 @@ def is_end_of_contract_day(date: datetime.datetime):
     return True if date.day == end_of_contract_day else False
 
 
-
 def load_all_shioaji_ticks(source_dir='../../shioaji_ticks'):
     all_files = []
 
@@ -505,11 +517,9 @@ def load_all_shioaji_ticks(source_dir='../../shioaji_ticks'):
 
 
 def build_dealer_downloader(date=datetime.datetime(year=2020, month=1, day=1), target_directory='自營商歷史資料'):
-
     if not os.path.isdir(target_directory):
         os.mkdir(target_directory)
 
-
     day_delta = datetime.timedelta(days=1)
     while date < datetime.datetime.now():
         print(date)
@@ -588,35 +598,89 @@ def build_dealer_downloader(date=datetime.datetime(year=2020, month=1, day=1), t
 
 
 
+class tplaysound(threading.Thread):
+
+    def __init__(self):
+        threading.Thread.__init__(self)
+
+        self.queue = queue.Queue()
+
+        self.sound_alert = f'{_MYPYLIB_ROOT}/sound/alert.wav'
+
+    def play_file(self, filename=None):
+        if filename is None:
+            filename = self.sound_alert
+        command = ('play', filename)
+        self.queue.put(command)
+        print(f'send command {command}')
+
+    def run(self):
+        while True:
+            try:
+                (cmd, arg) = self.queue.get(timeout=1)
+            except queue.Empty:
+                pass
+            else:
+                if cmd == 'stop':
+                    break
+
+                if cmd == 'play':
+                    print('got Play')
+                    if os.name == 'nt':
+                        print('NT play')
+                        winsound.PlaySound(arg, winsound.SND_FILENAME)
+                    else:
+                        print('POSIX play')
+                        playsound(arg)
+
+    def stop(self):
+        self.queue.put(('stop', 'stop'))
+
+
 
 
 if __name__ == '__main__':
     from time import sleep
 
-    build_dealer_downloader()
+    if True:
+        tps = tplaysound()
+        tps.start()
+        index = 0
+        while True:
+            tps.play_file()
+            tps.play_file()
+            tps.play_file()
+            tps.play_file()
+            sleep(3)
+            index += 1
+            if index == 3:
+                break
+        tps.stop()
+        tps.join()
 
-    exit(0)
+    if False:
+        build_dealer_downloader()
 
 
-    all = load_all_shioaji_ticks()
-    print(f'There are {len(all)} files')
+    if False:
+        all = load_all_shioaji_ticks()
+        print(f'There are {len(all)} files')
 
-    get_stock_future_snapshot()
-    get_stock_future_data()
+        get_stock_future_snapshot()
+        get_stock_future_data()
 
-    print(get_future_ex(datetime.datetime(year=2021, month=12, day=1)))
-    print(get_future_ex(datetime.datetime(year=2021, month=12, day=15)))
-    print(get_future_ex(datetime.datetime(year=2021, month=12, day=30)))
-    print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=1)))
-    print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=14)))
-    print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=15)))
-    print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=16)))
-    print(get_future_ex(datetime.datetime(year=2021, month=12, day=15), next_month=is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=15))))
+        print(get_future_ex(datetime.datetime(year=2021, month=12, day=1)))
+        print(get_future_ex(datetime.datetime(year=2021, month=12, day=15)))
+        print(get_future_ex(datetime.datetime(year=2021, month=12, day=30)))
+        print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=1)))
+        print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=14)))
+        print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=15)))
+        print(is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=16)))
+        print(get_future_ex(datetime.datetime(year=2021, month=12, day=15), next_month=is_end_of_contract_day(datetime.datetime(year=2021, month=12, day=15))))
 
-    print(get_ticks_between(100, 100))
+        print(get_ticks_between(100, 100))
 
-
-    mypylib_unit_test()
+        mypylib_unit_test()
 
     if False:
         for day in date_range(datetime.datetime(year=2019, month=1, day=1), datetime.datetime.today()):
@@ -627,6 +691,7 @@ if __name__ == '__main__':
                 get_short_selling_list(day)
                 sleep(5)
 
-    short_selling_to_csv()
+        short_selling_to_csv()
+
+        print(__LINE__())
 
-    print(__LINE__())
